        list p=16F628A, b=8, c= 102, n=71, t=on, st=off, f=inhx32
;******************************************************************
;*                                                                *
;*  Filename: 16F628A PWM 01.asm				  *
;*    Author: Mike McLaren, K8LH  (k8lh_at_arrl.net)		  *
;*      Date: 12-Jun-05  (last revision 12-Jun-05)		  *
;*								  *
;*   PWM / RC Integrator Test Program				  *
;*   								  *
;*   Using a 16F628A and 20-MHz crystal oscillator		  *
;*								  *
;*								  *
;*     MPLab: 7.11    (tabs=8)					  *
;*     MPAsm: 4.01						  *
;*								  *
;******************************************************************

	#include	<p16f628A.inc>
	errorlevel	-302

	__config	_PWRTE_ON&_LVP_OFF&_WDT_OFF&_HS_OSC&_MCLRE_OFF
;
;
PTRL	equ	0x2B		; PutString (main)
PTRH	equ	0x2C		; PutString (main)
TEMP	equ	0x2D		; PutByte (main)
;

;******************************************************************
;
;  _Title macro - home cursor, clear screen, print a string
;
_Title	macro	str		;
	local	String, Print
	movlw	low String	;
	movwf	PTRL		;
	movlw	high String	;
	movwf	PTRH		;
	goto	Print		;
String	dt	h'1B',"[2J"	; <esc>[2J sequence
	dt	str,0
Print	call	PutString	; print string
	endm

;******************************************************************
;*                                                             	  *
;*                                                                *
;*                                                                *
;*                                                                *
;*                                                                *
;******************************************************************

	org	0x0000

RESET	clrf	STATUS		;				  |B0
	clrf	PORTA		;				  |B0
	clrf	PORTB		;				  |B0
	goto	MAIN		;				  |B0

;******************************************************************
;*								  *
;*  Init232 subroutine						  *
;*								  *
;******************************************************************
;
;  initialize RS-232 peripheral
;
Init232	bsf	STATUS,RP0	; bank 1			  |B1
	bsf	TRISB,1		; set RB1/RX as input		  |B1
	bsf	TRISB,2		; set RB2/TX as input		  |B1
	movlw	d'129'		; 9600 baud (20-MHz, BRGH=1)	  |B1
	movwf	SPBRG		;				  |B1
	movlw	b'00100100'	; TXEN=1, SYNC=0, BRGH=1	  |B1
	movwf	TXSTA		;				  |B1
	bcf	STATUS,RP0	; bank 0			  |B0
	movlw	b'10010000'	; SPEN=1, CREN=1		  |B0
	movwf	RCSTA		;				  |B0
	movf	RCREG,W		; flush RS232 rx buffer		  |B0
	movf	RCREG,W		;				  |B0
	return			;				  |B0

;******************************************************************
;
;  Put232 - enter with character to be sent in W
;
Put232	btfss	PIR1,TXIF	; TX in progress?		  |B0
	goto	Put232		; yes, branch and wait		  |B0
	movwf	TXREG		; send character		  |B0
	return			;				  |B0

;******************************************************************
;
;  Get232 - exit with received character in W
;
Get232	btfss	PIR1,RCIF	; character available flag?	  |B0
	goto	Get232		; no, branch and wait		  |B0
	movf	RCREG,W		; yes, get character		  |B0
	return			;				  |B0

;******************************************************************
;
;  Print String - enter with string address in PTRL & PTRH
;		- string must be terminated with a '0' byte
;
PutString
	call	GetTable	; get a table character		  |B0
	andlw	b'11111111'	;				  |B0
	btfsc	STATUS,Z	; last character?		  |B0
	return			; yes, return			  |B0
	call	Put232		; output char			  |B0
	incfsz	PTRL,F		; increment pointer		  |B0
	goto	PutString	;				  |B0
	incf	PTRH,F		;				  |B0
	goto	PutString	;				  |B0
;
GetTable
	movf	PTRH,W		;				  |B0
	movwf	PCLATH		;				  |B0
	movf	PTRL,W		;				  |B0
	movwf	PCL		;				  |B0

;******************************************************************
;
;  Print byte in W as two ASCII nybbles
;
PutByte	movwf	TEMP		; save byte			  |B0
	swapf	TEMP,W		; swap nibbles in W		  |B0
	call	Hex2Asc		; process left nibble		  |B0
	movf	TEMP,W		; process right nibble		  |B0
Hex2Asc	andlw	b'00001111'	; mask off left nibble		  |B0
	addlw	h'36'		;				  |B0
	btfsc	STATUS,DC	;				  |B0
	addlw	h'07'		;				  |B0
	addlw	0-6		; ($FA)				  |B0
	goto	Put232		; print ASCII nibble		  |B0

;******************************************************************
;
;  setup PWM
;
InitPWM
	bcf	STATUS,RP0	;				  |B0
	movlw	b'00000010'	;				  |B0
	movwf	T2CON		; prescale:16 (3.2-usec ticks)	  |B0
	movlw	d'40'		; 40 3.2-usec 'ticks'		  |B0
	movwf	CCPR1L		; 80% duty cycle		  |B0
	bsf	STATUS,RP0	; select bank 1			  |B1
	movlw	d'50'-1		; 50 3.2-usec 'ticks'		  |B1
	movwf	PR2		; Period:160-usecs (6.25 KHz)	  |B1
	bcf	TRISB,3		; make RB3/CCP1 pin output	  |B1
	bcf	STATUS,RP0	; select bank 0			  |B0
	movlw	b'00001100'	;				  |B0
	movwf	CCP1CON		; put CCP module in PWM mode	  |B0
	bsf	T2CON,TMR2ON	; turn on TMR2			  |B0
	return			;				  |B0

;******************************************************************
;*								  *
;*  								  *
;*								  *
;******************************************************************

MAIN	movlw	h'07'		;				  |B0
	movwf	CMCON		; turn comparator off		  |B0
	bsf	STATUS,RP0	; bank 1			  |B1
	clrf	TRISA		; PORTA all outputs		  |B1
	clrf	TRISB		; PORTB all outputs		  |B1
	bcf	STATUS,RP0	; bank 0			  |B0
;
;  initialize Serial I/O
;
	call	Init232		;				  |B0
;
;  initialize PWM module
;
	call	InitPWM		;				  |B0
;
;  _Title macro will home cursor, clear screen, and print string
;
	_Title	"K8LH 16F628A RC Integrator Experiment\r\n\n"
;

LOOP	movlw	h'0D'		; the <cr> char			  |B0
	call	Put232		;				  |B0
	movf	CCPR1L,W	;				  |B0
	call	PutByte		;				  |B0
L1	call	Get232		; get a receive character	  |B0
	movwf	TEMP		; save it			  |B0
	xorlw	a'<'		; the "decrease" key?		  |B0
	bz	Dec_PWM		; yes, branch, else		  |B0
	xorlw	a'>'^a'<'	; the "increase" key?		  |B0
	bz	Inc_PWM		; yes, branch, else		  |B0
	movf	TEMP,W		;				  |B0
	call	Put232		; echo it back to sender	  |B0
	goto	L1		; loop forever			  |B0
;
Dec_PWM	movf	CCPR1L,W	;				  |B0
	skpz			;				  |B0
	decf	CCPR1L,f	;				  |B0
	goto	LOOP		;				  |B0
;
Inc_PWM	movf	CCPR1L,W	;				  |B0
	xorlw	d'50'		; upper limit?			  |B0
	skpz			; yes, skip			  |B0
	incf	CCPR1L,f	; else increment		  |B0
	goto	LOOP		;				  |B0

;******************************************************************
	end

